#include <YoggieEngine.h>
#include "Window.h"
namespace YoggieEngine {
	void LoadGLExtensions();
	
	NativeWindow::NativeWindow(const int width, const int height) :
		Width(width), Height(height), FullScreen(false)
	{
		if (InitGLFW() == false) {
			exit(-1);
		}

		glfwWindowHint(GLFW_DOUBLEBUFFER, GLFW_TRUE);
		glfwWindowHint(GLFW_FOCUS_ON_SHOW, GLFW_TRUE);
		
		if( FullScreen )
			glfwWindowHint(GLFW_MAXIMIZED, GLFW_TRUE);
		if( !Resizable)
			glfwWindowHint(GLFW_RESIZABLE, GL_FALSE);
		if (!VSync)
			glfwSwapInterval(0);

		window = glfwCreateWindow(Width, Height, "BarinkEngine", NULL, NULL);

		if (!window)
		{
			spdlog::error("GLFW failed to create window!");
			glfwTerminate();
			return;
		}

		SetContext();
		glfwGetFramebufferSize(window, &Width, &Height);
		LoadGLExtensions();

	}

	void NativeWindow::SetViewPort(int width, int height) {
		glViewport(0, 0, width, height);
	}

	NativeWindow::~NativeWindow() {
		glfwTerminate();
	}

	bool NativeWindow::WindowShouldClose() {
		return glfwWindowShouldClose(window);
	}

	void NativeWindow::SetContext()
	{
		glfwMakeContextCurrent(window);
	}

	void NativeWindow::Poll()
	{
		glfwPollEvents();
	}

	void NativeWindow::SwapBuffers()
	{
		glfwSwapBuffers(window);
	}

	bool NativeWindow::InitGLFW() 
	{
		if (!glfwInit())
		{
			spdlog::error("Failed to initialise GLFW!");
			return false;
		}

		return true;
	}

	void LoadGLExtensions()
	{
		if (!gladLoadGLLoader((GLADloadproc)glfwGetProcAddress)) {
			spdlog::error("Failed to initialize GLAD!\n");
			exit(-1);
		}
	}
	


}
